// Copyright (C) 2021 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only

#ifndef QTMULTIMEDIAWIDGETSGLOBAL_H
#define QTMULTIMEDIAWIDGETSGLOBAL_H

#include <QtMultimediaWidgets/qtmultimediawidgetsexports.h>

#endif // QTMULTIMEDIAWIDGETSGLOBAL_H
