// Copyright (C) 2026 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only

#ifndef QTMULTIMEDIAQUICKGLOBAL_H
#define QTMULTIMEDIAQUICKGLOBAL_H

#include <QtMultimedia/qtmultimediaglobal.h>
#include <QtQuick/qtquickglobal.h>
#include <QtMultimediaQuick/qtmultimediaquickexports.h>

#endif // QTMULTIMEDIAQUICKGLOBAL_H
